@@ -1,14 +1,15 @@
 """
 Helper functions for mapping model fields to a dictionary of default
-keyword arguments that should be used for their equivelent serializer fields.
+keyword arguments that should be used for their equivalent serializer fields.
 """
+import inspect
+
 from django.core import validators
 from django.db import models
 from django.utils.text import capfirst
-from rest_framework.compat import clean_manytomany_helptext
-from rest_framework.validators import UniqueValidator
-import inspect
 
+from rest_framework.compat import DecimalValidator, JSONField
+from rest_framework.validators import UniqueValidator
 
 NUMERIC_FIELD_TYPES = (
     models.IntegerField, models.FloatField, models.DecimalField
@@ -36,7 +37,10 @@ class ClassLookupDict(object):
         for cls in inspect.getmro(base_class):
             if cls in self.mapping:
                 return self.mapping[cls]
-        raise KeyError('Class %s not found in lookup.', cls.__name__)
+        raise KeyError('Class %s not found in lookup.' % base_class.__name__)
+
+    def __setitem__(self, key, value):
+        self.mapping[key] = value
 
 
 def needs_label(model_field, field_name):
@@ -84,7 +88,7 @@ def get_field_kwargs(field_name, model_field):
     if decimal_places is not None:
         kwargs['decimal_places'] = decimal_places
 
-    if isinstance(model_field, models.TextField):
+    if isinstance(model_field, models.TextField) or (JSONField and isinstance(model_field, JSONField)):
         kwargs['style'] = {'base_template': 'textarea.html'}
 
     if isinstance(model_field, models.AutoField) or not model_field.editable:
@@ -99,19 +103,97 @@ def get_field_kwargs(field_name, model_field):
     if model_field.null and not isinstance(model_field, models.NullBooleanField):
         kwargs['allow_null'] = True
 
-    if model_field.blank:
+    if model_field.blank and (isinstance(model_field, models.CharField) or
+                              isinstance(model_field, models.TextField)):
         kwargs['allow_blank'] = True
 
-    if model_field.flatchoices:
-        # If this model field contains choices, then return early.
-        # Further keyword arguments are not valid.
-        kwargs['choices'] = model_field.flatchoices
-        return kwargs
+    if isinstance(model_field, models.FilePathField):
+        kwargs['path'] = model_field.path
+
+        if model_field.match is not None:
+            kwargs['match'] = model_field.match
+
+        if model_field.recursive is not False:
+            kwargs['recursive'] = model_field.recursive
+
+        if model_field.allow_files is not True:
+            kwargs['allow_files'] = model_field.allow_files
+
+        if model_field.allow_folders is not False:
+            kwargs['allow_folders'] = model_field.allow_folders
+
+    if model_field.choices:
+        kwargs['choices'] = model_field.choices
+    else:
+        # Ensure that max_value is passed explicitly as a keyword arg,
+        # rather than as a validator.
+        max_value = next((
+            validator.limit_value for validator in validator_kwarg
+            if isinstance(validator, validators.MaxValueValidator)
+        ), None)
+        if max_value is not None and isinstance(model_field, NUMERIC_FIELD_TYPES):
+            kwargs['max_value'] = max_value
+            validator_kwarg = [
+                validator for validator in validator_kwarg
+                if not isinstance(validator, validators.MaxValueValidator)
+            ]
+
+        # Ensure that min_value is passed explicitly as a keyword arg,
+        # rather than as a validator.
+        min_value = next((
+            validator.limit_value for validator in validator_kwarg
+            if isinstance(validator, validators.MinValueValidator)
+        ), None)
+        if min_value is not None and isinstance(model_field, NUMERIC_FIELD_TYPES):
+            kwargs['min_value'] = min_value
+            validator_kwarg = [
+                validator for validator in validator_kwarg
+                if not isinstance(validator, validators.MinValueValidator)
+            ]
+
+        # URLField does not need to include the URLValidator argument,
+        # as it is explicitly added in.
+        if isinstance(model_field, models.URLField):
+            validator_kwarg = [
+                validator for validator in validator_kwarg
+                if not isinstance(validator, validators.URLValidator)
+            ]
+
+        # EmailField does not need to include the validate_email argument,
+        # as it is explicitly added in.
+        if isinstance(model_field, models.EmailField):
+            validator_kwarg = [
+                validator for validator in validator_kwarg
+                if validator is not validators.validate_email
+            ]
+
+        # SlugField do not need to include the 'validate_slug' argument,
+        if isinstance(model_field, models.SlugField):
+            validator_kwarg = [
+                validator for validator in validator_kwarg
+                if validator is not validators.validate_slug
+            ]
+
+        # IPAddressField do not need to include the 'validate_ipv46_address' argument,
+        if isinstance(model_field, models.GenericIPAddressField):
+            validator_kwarg = [
+                validator for validator in validator_kwarg
+                if validator is not validators.validate_ipv46_address
+            ]
+        # Our decimal validation is handled in the field code, not validator code.
+        # (In Django 1.9+ this differs from previous style)
+        if isinstance(model_field, models.DecimalField) and DecimalValidator:
+            validator_kwarg = [
+                validator for validator in validator_kwarg
+                if not isinstance(validator, DecimalValidator)
+            ]
 
     # Ensure that max_length is passed explicitly as a keyword arg,
     # rather than as a validator.
     max_length = getattr(model_field, 'max_length', None)
-    if max_length is not None and isinstance(model_field, models.CharField):
+    if max_length is not None and (isinstance(model_field, models.CharField) or
+                                   isinstance(model_field, models.TextField) or
+                                   isinstance(model_field, models.FileField)):
         kwargs['max_length'] = max_length
         validator_kwarg = [
             validator for validator in validator_kwarg
@@ -131,57 +213,16 @@ def get_field_kwargs(field_name, model_field):
             if not isinstance(validator, validators.MinLengthValidator)
         ]
 
-    # Ensure that max_value is passed explicitly as a keyword arg,
-    # rather than as a validator.
-    max_value = next((
-        validator.limit_value for validator in validator_kwarg
-        if isinstance(validator, validators.MaxValueValidator)
-    ), None)
-    if max_value is not None and isinstance(model_field, NUMERIC_FIELD_TYPES):
-        kwargs['max_value'] = max_value
-        validator_kwarg = [
-            validator for validator in validator_kwarg
-            if not isinstance(validator, validators.MaxValueValidator)
-        ]
-
-    # Ensure that max_value is passed explicitly as a keyword arg,
-    # rather than as a validator.
-    min_value = next((
-        validator.limit_value for validator in validator_kwarg
-        if isinstance(validator, validators.MinValueValidator)
-    ), None)
-    if min_value is not None and isinstance(model_field, NUMERIC_FIELD_TYPES):
-        kwargs['min_value'] = min_value
-        validator_kwarg = [
-            validator for validator in validator_kwarg
-            if not isinstance(validator, validators.MinValueValidator)
-        ]
-
-    # URLField does not need to include the URLValidator argument,
-    # as it is explicitly added in.
-    if isinstance(model_field, models.URLField):
-        validator_kwarg = [
-            validator for validator in validator_kwarg
-            if not isinstance(validator, validators.URLValidator)
-        ]
-
-    # EmailField does not need to include the validate_email argument,
-    # as it is explicitly added in.
-    if isinstance(model_field, models.EmailField):
-        validator_kwarg = [
-            validator for validator in validator_kwarg
-            if validator is not validators.validate_email
-        ]
-
-    # SlugField do not need to include the 'validate_slug' argument,
-    if isinstance(model_field, models.SlugField):
-        validator_kwarg = [
-            validator for validator in validator_kwarg
-            if validator is not validators.validate_slug
-        ]
-
     if getattr(model_field, 'unique', False):
-        validator = UniqueValidator(queryset=model_field.model._default_manager)
+        unique_error_message = model_field.error_messages.get('unique', None)
+        if unique_error_message:
+            unique_error_message = unique_error_message % {
+                'model_name': model_field.model._meta.verbose_name,
+                'field_label': model_field.verbose_name
+            }
+        validator = UniqueValidator(
+            queryset=model_field.model._default_manager,
+            message=unique_error_message)
         validator_kwarg.append(validator)
 
     if validator_kwarg:
@@ -194,7 +235,7 @@ def get_relation_kwargs(field_name, relation_info):
     """
     Creates a default instance of a flat relational field.
     """
-    model_field, related_model, to_many, has_through_model = relation_info
+    model_field, related_model, to_many, to_field, has_through_model, reverse = relation_info
     kwargs = {
         'queryset': related_model._default_manager,
         'view_name': get_detail_view_name(related_model)
@@ -203,6 +244,9 @@ def get_relation_kwargs(field_name, relation_info):
     if to_many:
         kwargs['many'] = True
 
+    if to_field:
+        kwargs['to_field'] = to_field
+
     if has_through_model:
         kwargs['read_only'] = True
         kwargs.pop('queryset', None)
@@ -210,7 +254,7 @@ def get_relation_kwargs(field_name, relation_info):
     if model_field:
         if model_field.verbose_name and needs_label(model_field, field_name):
             kwargs['label'] = capfirst(model_field.verbose_name)
-        help_text = clean_manytomany_helptext(model_field.help_text)
+        help_text = model_field.help_text
         if help_text:
             kwargs['help_text'] = help_text
         if not model_field.editable:
@@ -220,7 +264,8 @@ def get_relation_kwargs(field_name, relation_info):
             # If this field is read-only, then return early.
             # No further keyword arguments are valid.
             return kwargs
-        if model_field.has_default() or model_field.null:
+
+        if model_field.has_default() or model_field.blank or model_field.null:
             kwargs['required'] = False
         if model_field.null:
             kwargs['allow_null'] = True
@@ -229,6 +274,8 @@ def get_relation_kwargs(field_name, relation_info):
         if getattr(model_field, 'unique', False):
             validator = UniqueValidator(queryset=model_field.model._default_manager)
             kwargs['validators'] = kwargs.get('validators', []) + [validator]
+        if to_many and not model_field.blank:
+            kwargs['allow_empty'] = False
 
     return kwargs
 
